package org.richfaces.renderkit;

import java.io.IOException;
import java.util.*;
import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.*;
import org.ajax4jsf.javascript.*;
import org.ajax4jsf.model.*;
import org.richfaces.component.*;
import org.richfaces.component.util.HtmlUtil;
import org.richfaces.context.OnOffResponseWriter;
import org.richfaces.model.SelectionMode;
import org.richfaces.renderkit.util.AjaxRendererUtils;
import org.richfaces.renderkit.util.RendererUtils;

// Referenced classes of package org.richfaces.renderkit:
//            SelectionRenderer, ComponentAttribute, MetaComponentRenderer, RenderKitUtils, 
//            AjaxFunction, AjaxOptions, RowHolderBase

public class ExtendedDataTableRenderer extends SelectionRenderer
    implements MetaComponentRenderer
{
    private static abstract class EncoderVariance extends Enum
    {

        public static final EncoderVariance full;
        public static final EncoderVariance partial;
        private static final EncoderVariance $VALUES[];

        public static EncoderVariance[] values()
        {
            return (EncoderVariance[])$VALUES.clone();
        }

        public static EncoderVariance valueOf(String name)
        {
            return (EncoderVariance)Enum.valueOf(org/richfaces/renderkit/ExtendedDataTableRenderer$EncoderVariance, name);
        }

        public abstract void encodeStartUpdate(FacesContext facescontext, String s)
            throws IOException;

        public abstract void encodeEndUpdate(FacesContext facescontext)
            throws IOException;

        static 
        {
            full = new EncoderVariance("full", 0) {

                public void encodeStartUpdate(FacesContext facescontext, String s)
                    throws IOException
                {
                }

                public void encodeEndUpdate(FacesContext facescontext)
                    throws IOException
                {
                }

            };
            partial = new EncoderVariance("partial", 1) {

                private void switchResponseWriter(FacesContext context, boolean writerState)
                {
                    ResponseWriter writer = context.getResponseWriter();
                    ((OnOffResponseWriter)writer).setSwitchedOn(writerState);
                }

                public void encodeStartUpdate(FacesContext context, String targetId)
                    throws IOException
                {
                    switchResponseWriter(context, true);
                    context.getPartialViewContext().getPartialResponseWriter().startUpdate(targetId);
                }

                public void encodeEndUpdate(FacesContext context)
                    throws IOException
                {
                    context.getPartialViewContext().getPartialResponseWriter().endUpdate();
                    switchResponseWriter(context, false);
                }

            };
            $VALUES = (new EncoderVariance[] {
                full, partial
            });
        }

        private EncoderVariance(String s, int i)
        {
            super(s, i);
        }

    }

    private class RendererState extends RowHolderBase
    {

        private UIDataTableBase table;
        private List parts;
        private Part current;
        private Iterator partIterator;
        private EncoderVariance encoderVariance;
        final ExtendedDataTableRenderer this$0;

        public UIDataTableBase getRow()
        {
            return table;
        }

        public void startIterate()
        {
            partIterator = parts.iterator();
        }

        public Part nextPart()
        {
            current = (Part)partIterator.next();
            return current;
        }

        public Part getPart()
        {
            return current;
        }

        public boolean hasNextPart()
        {
            return partIterator.hasNext();
        }

        public EncoderVariance getEncoderVariance()
        {
            return encoderVariance;
        }

        public void setEncoderVariance(EncoderVariance encoderVariance)
        {
            this.encoderVariance = encoderVariance;
        }

        public volatile Row getRow()
        {
            return getRow();
        }

        public RendererState(FacesContext context, UIDataTableBase table)
        {
            this$0 = ExtendedDataTableRenderer.this;
            super(context);
            encoderVariance = EncoderVariance.full;
            this.table = table;
            List columns = new ArrayList();
            Iterator iterator = table.columns();
            Map columnsMap = new LinkedHashMap();
            do
            {
                if(!iterator.hasNext())
                {
                    break;
                }
                UIComponent component = (UIComponent)iterator.next();
                if(component.isRendered())
                {
                    columnsMap.put(component.getId(), component);
                }
            } while(true);
            String columnsOrder[] = (String[])(String[])table.getAttributes().get("columnsOrder");
            if(columnsOrder != null && columnsOrder.length > 0)
            {
                for(int i = 0; i < columnsOrder.length && !columnsMap.isEmpty(); i++)
                {
                    columns.add(columnsMap.remove(columnsOrder[i]));
                }

            }
            for(iterator = columnsMap.values().iterator(); iterator.hasNext(); columns.add(iterator.next())) { }
            int frozenColumnsAttribute = ((Integer)table.getAttributes().get("frozenColumns")).intValue();
            if(frozenColumnsAttribute < 0)
            {
                frozenColumnsAttribute = 0;
            }
            int count = Math.min(frozenColumnsAttribute, columns.size());
            List frozenColumns = columns.subList(0, count);
            columns = columns.subList(count, columns.size());
            parts = new ArrayList(PartName.values().length);
            if(frozenColumns.size() > 0)
            {
                parts.add(new Part(PartName.frozen, frozenColumns));
            }
            if(columns.size() > 0)
            {
                parts.add(new Part(PartName.normal, columns));
            }
        }
    }

    private class Part
    {

        private PartName name;
        private List columns;
        final ExtendedDataTableRenderer this$0;

        public PartName getName()
        {
            return name;
        }

        public List getColumns()
        {
            return columns;
        }

        public Part(PartName name, List columns)
        {
            this$0 = ExtendedDataTableRenderer.this;
            super();
            this.name = name;
            this.columns = columns;
        }
    }

    private static final class PartName extends Enum
    {

        public static final PartName frozen;
        public static final PartName normal;
        private String id;
        private static final PartName $VALUES[];

        public static PartName[] values()
        {
            return (PartName[])$VALUES.clone();
        }

        public static PartName valueOf(String name)
        {
            return (PartName)Enum.valueOf(org/richfaces/renderkit/ExtendedDataTableRenderer$PartName, name);
        }

        public String getId()
        {
            return id;
        }

        static 
        {
            frozen = new PartName("frozen", 0);
            normal = new PartName("normal", 1);
            $VALUES = (new PartName[] {
                frozen, normal
            });
        }

        private PartName(String s, int i)
        {
            super(s, i);
            id = String.valueOf(toString().charAt(0));
        }
    }


    private static final JSReference CLIENT_PARAMS = new JSReference("clientParams");
    private static final Map EVENT_ATTRIBUTES = Collections.unmodifiableMap(ComponentAttribute.createMap(new ComponentAttribute[] {
        (new ComponentAttribute("onselectionchange")).setEventNames(new String[] {
            "selectionchange"
        }), (new ComponentAttribute("onbeforeselectionchange")).setEventNames(new String[] {
            "beforeselectionchange"
        })
    }));
    static final boolean $assertionsDisabled = !org/richfaces/renderkit/ExtendedDataTableRenderer.desiredAssertionStatus();

    public ExtendedDataTableRenderer()
    {
    }

    private void encodeEmptyFooterCell(FacesContext context, ResponseWriter writer, UIComponent column)
        throws IOException
    {
        if(column.isRendered())
        {
            writer.startElement("td", column);
            writer.startElement("div", column);
            writer.writeAttribute("class", (new StringBuilder()).append("rf-edt-ftr-c-emp rf-edt-c-").append(column.getId()).toString(), null);
            writer.endElement("div");
            writer.endElement("td");
        }
    }

    private void encodeHeaderOrFooterCell(FacesContext context, ResponseWriter writer, UIComponent column, String facetName)
        throws IOException
    {
        if(column.isRendered())
        {
            String classAttribute = (new StringBuilder()).append(facetName).append("Class").toString();
            writer.startElement("td", column);
            if("header".equals(facetName))
            {
                writer.startElement("div", column);
                writer.writeAttribute("class", (new StringBuilder()).append("rf-edt-rsz-cntr rf-edt-c-").append(column.getId()).toString(), null);
                writer.startElement("div", column);
                writer.writeAttribute("class", "rf-edt-rsz", null);
                writer.endElement("div");
                writer.endElement("div");
            }
            writer.startElement("div", column);
            writer.writeAttribute("class", HtmlUtil.concatClasses(new Object[] {
                (new StringBuilder()).append("rf-edt-").append(getFacetClassName(facetName)).append("-c").toString(), (new StringBuilder()).append("rf-edt-c-").append(column.getId()).toString(), (String)column.getAttributes().get(classAttribute)
            }), null);
            writer.startElement("div", column);
            writer.writeAttribute("class", (new StringBuilder()).append("rf-edt-").append(getFacetClassName(facetName)).append("-c-cnt").toString(), null);
            UIComponent facet = column.getFacet(facetName);
            if(facet != null && facet.isRendered())
            {
                facet.encodeAll(context);
            }
            writer.endElement("div");
            writer.endElement("div");
            writer.endElement("td");
        }
    }

    private String getFacetClassName(String name)
    {
        if("header".equals(name))
        {
            return "hdr";
        }
        if("footer".equals(name))
        {
            return "ftr";
        } else
        {
            throw new IllegalArgumentException(name);
        }
    }

    private void encodeHeaderOrFooter(RendererState state, String name)
        throws IOException
    {
        FacesContext context = state.getContext();
        ResponseWriter writer = context.getResponseWriter();
        UIDataTableBase table = state.getRow();
        boolean columnFacetPresent = table.isColumnFacetPresent(name);
        if(columnFacetPresent || "footer".equals(name))
        {
            writer.startElement("div", table);
            writer.writeAttribute("class", (new StringBuilder()).append("rf-edt-").append(getFacetClassName(name)).toString(), null);
            writer.startElement("table", table);
            writer.writeAttribute("cellpadding", "0", null);
            writer.writeAttribute("cellspacing", "0", null);
            writer.startElement("tbody", table);
            writer.startElement("tr", table);
            String clientId = table.getClientId(context);
            state.startIterate();
            do
            {
                if(!state.hasNextPart())
                {
                    break;
                }
                Part part = state.nextPart();
                PartName partName = part.getName();
                Iterator columns = part.getColumns().iterator();
                if(columns.hasNext())
                {
                    writer.startElement("td", table);
                    if(PartName.frozen.equals(partName) && "footer".equals(name))
                    {
                        writer.writeAttribute("class", "rf-edt-ftr-fzn", null);
                    }
                    writer.startElement("div", table);
                    if(PartName.frozen.equals(partName))
                    {
                        if("header".equals(name))
                        {
                            writer.writeAttribute("id", (new StringBuilder()).append(clientId).append(":frozenHeader").toString(), null);
                        }
                    } else
                    {
                        writer.writeAttribute("id", (new StringBuilder()).append(clientId).append(":").append(name).toString(), null);
                        writer.writeAttribute("class", (new StringBuilder()).append("rf-edt-cnt").append("footer".equals(name) ? " rf-edt-ftr-cnt" : "").toString(), null);
                    }
                    String tableId = (new StringBuilder()).append(clientId).append(":cf").append(name.charAt(0)).append(partName.getId()).toString();
                    EncoderVariance encoderVariance = state.getEncoderVariance();
                    encoderVariance.encodeStartUpdate(context, tableId);
                    writer.startElement("table", table);
                    writer.writeAttribute("id", tableId, null);
                    writer.writeAttribute("cellpadding", "0", null);
                    writer.writeAttribute("cellspacing", "0", null);
                    writer.startElement("tbody", table);
                    writer.startElement("tr", table);
                    while(columns.hasNext()) 
                    {
                        if(columnFacetPresent)
                        {
                            encodeHeaderOrFooterCell(context, writer, (UIComponent)columns.next(), name);
                        } else
                        {
                            encodeEmptyFooterCell(context, writer, (UIComponent)columns.next());
                        }
                    }
                    writer.endElement("tr");
                    writer.endElement("tbody");
                    writer.endElement("table");
                    encoderVariance.encodeEndUpdate(context);
                    writer.endElement("div");
                    writer.endElement("td");
                }
            } while(true);
            writer.endElement("tr");
            writer.endElement("tbody");
            writer.endElement("table");
            writer.endElement("div");
        }
    }

    public void encodeHeader(RendererState state)
        throws IOException
    {
        FacesContext context = state.getContext();
        ResponseWriter writer = context.getResponseWriter();
        UIDataTableBase table = state.getRow();
        UIComponent header = table.getFacet("header");
        if(header != null && header.isRendered())
        {
            String elementId = (new StringBuilder()).append(table.getClientId(context)).append(":tfh").toString();
            EncoderVariance encoderVariance = state.getEncoderVariance();
            encoderVariance.encodeStartUpdate(context, elementId);
            writer.startElement("div", table);
            writer.writeAttribute("id", elementId, null);
            writer.writeAttribute("class", HtmlUtil.concatClasses(new Object[] {
                "rf-edt-tbl-hdr", table.getHeaderClass()
            }), null);
            header.encodeAll(context);
            writer.endElement("div");
            encoderVariance.encodeEndUpdate(context);
        }
        encodeHeaderOrFooter(state, "header");
    }

    public void encodeBody(RendererState state)
        throws IOException
    {
        FacesContext context = state.getContext();
        ResponseWriter writer = context.getResponseWriter();
        UIDataTableBase table = state.getRow();
        String clientId = table.getClientId(context);
        String tableBodyId = (new StringBuilder()).append(clientId).append(":b").toString();
        EncoderVariance encoderVariance = state.getEncoderVariance();
        encoderVariance.encodeStartUpdate(context, tableBodyId);
        writer.startElement("div", table);
        writer.writeAttribute("id", tableBodyId, null);
        writer.writeAttribute("class", "rf-edt-b", null);
        if(table.getRowCount() == 0)
        {
            UIComponent facet = table.getFacet("noData");
            writer.startElement("div", table);
            writer.writeAttribute("class", "rf-edt-ndt", null);
            if(facet != null && facet.isRendered())
            {
                facet.encodeAll(context);
            } else
            {
                Object noDataLabel = table.getAttributes().get("noDataLabel");
                if(noDataLabel != null)
                {
                    writer.writeText(noDataLabel, "noDataLabel");
                }
            }
            writer.endElement("div");
        } else
        {
            table.getAttributes().put("clientFirst", Integer.valueOf(0));
            writer.startElement("div", table);
            writer.startElement("div", table);
            writer.writeAttribute("class", "rf-edt-spcr", null);
            writer.endElement("div");
            writer.startElement("table", table);
            writer.writeAttribute("cellpadding", "0", null);
            writer.writeAttribute("cellspacing", "0", null);
            writer.startElement("tbody", table);
            writer.startElement("tr", table);
            state.startIterate();
            for(; state.hasNextPart(); writer.endElement("td"))
            {
                writer.startElement("td", table);
                writer.startElement("div", table);
                PartName partName = state.nextPart().getName();
                if(PartName.normal.equals(partName))
                {
                    writer.writeAttribute("id", (new StringBuilder()).append(clientId).append(":body").toString(), null);
                    writer.writeAttribute("class", "rf-edt-cnt", null);
                }
                String targetId = (new StringBuilder()).append(clientId).append(":tbt").append(partName.getId()).toString();
                writer.startElement("table", table);
                writer.writeAttribute("id", targetId, null);
                writer.writeAttribute("cellpadding", "0", null);
                writer.writeAttribute("cellspacing", "0", null);
                writer.startElement("tbody", table);
                writer.writeAttribute("id", (new StringBuilder()).append(clientId).append(":tb").append(partName.getId()).toString(), null);
                encodeRows(context, state);
                writer.endElement("tbody");
                writer.endElement("table");
                writer.endElement("div");
            }

            writer.endElement("tr");
            writer.endElement("tbody");
            writer.endElement("table");
            writer.endElement("div");
        }
        writer.endElement("div");
        encoderVariance.encodeEndUpdate(context);
    }

    public void encodeFooter(RendererState state)
        throws IOException
    {
        FacesContext context = state.getContext();
        ResponseWriter writer = context.getResponseWriter();
        UIDataTableBase table = state.getRow();
        encodeHeaderOrFooter(state, "footer");
        UIComponent footer = table.getFacet("footer");
        if(footer != null && footer.isRendered())
        {
            String elementId = (new StringBuilder()).append(table.getClientId(context)).append(":tff").toString();
            EncoderVariance encoderVariance = state.getEncoderVariance();
            encoderVariance.encodeStartUpdate(context, elementId);
            writer.startElement("div", table);
            writer.writeAttribute("id", elementId, null);
            writer.writeAttribute("class", HtmlUtil.concatClasses(new Object[] {
                "rf-edt-tbl-ftr", table.getFooterClass()
            }), null);
            footer.encodeAll(context);
            writer.endElement("div");
            encoderVariance.encodeEndUpdate(context);
        }
    }

    protected Class getComponentClass()
    {
        return org/richfaces/component/AbstractExtendedDataTable;
    }

    public void encodeMetaComponent(FacesContext context, UIComponent component, String metaComponentId)
        throws IOException
    {
        AbstractExtendedDataTable table;
        ResponseWriter initialWriter;
        table = (AbstractExtendedDataTable)component;
        if("scroll".equals(metaComponentId))
        {
            final PartialResponseWriter writer = context.getPartialViewContext().getPartialResponseWriter();
            int clientFirst = table.getClientFirst();
            Integer oldClientFirst = (Integer)table.getAttributes().remove("oldClientFirst");
            if(oldClientFirst == null)
            {
                oldClientFirst = Integer.valueOf(clientFirst);
            }
            int clientRows = ((SequenceRange)table.getComponentState().getRange()).getRows();
            int difference = clientFirst - oldClientFirst.intValue();
            SequenceRange addRange = null;
            SequenceRange removeRange = null;
            if(Math.abs(difference) >= clientRows)
            {
                difference = 0;
                addRange = new SequenceRange(clientFirst, clientRows);
                removeRange = new SequenceRange(oldClientFirst.intValue(), clientRows);
            } else
            if(difference < 0)
            {
                clientFirst += table.getFirst();
                addRange = new SequenceRange(clientFirst, -difference);
                removeRange = new SequenceRange(clientFirst + clientRows, -difference);
            } else
            if(difference > 0)
            {
                oldClientFirst = Integer.valueOf(oldClientFirst.intValue() + table.getFirst());
                removeRange = new SequenceRange(oldClientFirst.intValue(), difference);
                int last = oldClientFirst.intValue() + clientRows;
                addRange = new SequenceRange(last, difference);
            }
            if(addRange != null)
            {
                Object key = table.getRowKey();
                table.captureOrigValue(context);
                table.setRowKey(context, null);
                final RendererState state = createRowHolder(context, table, null);
                String clientId = table.getClientId(context);
                state.startIterate();
                for(; state.hasNextPart(); writer.endEval())
                {
                    String partId = state.nextPart().getName().getId();
                    final List ids = new LinkedList();
                    table.walk(context, new DataVisitor() {

                        final RendererState val$state;
                        final List val$ids;
                        final ExtendedDataTableRenderer this$0;

                        public DataVisitResult process(FacesContext context, Object rowKey, Object argument)
                        {
                            UIDataTableBase dataTable = state.getRow();
                            dataTable.setRowKey(context, rowKey);
                            ids.add((new StringBuilder()).append(dataTable.getContainerClientId(context)).append(":").append(state.getPart().getName().getId()).toString());
                            return DataVisitResult.CONTINUE;
                        }

            
            {
                this$0 = ExtendedDataTableRenderer.this;
                state = rendererstate;
                ids = list;
                super();
            }
                    }, removeRange, null);
                    table.walk(context, new DataVisitor() {

                        final RendererState val$state;
                        final PartialResponseWriter val$writer;
                        final List val$ids;
                        final ExtendedDataTableRenderer this$0;

                        public DataVisitResult process(FacesContext context, Object rowKey, Object argument)
                        {
                            UIDataTableBase dataTable = state.getRow();
                            dataTable.setRowKey(context, rowKey);
                            HashMap attributes = new HashMap(1);
                            String id = (new StringBuilder()).append(dataTable.getContainerClientId(context)).append(":").append(state.getPart().getName().getId()).toString();
                            attributes.put("id", id);
                            try
                            {
                                writer.updateAttributes((String)ids.remove(0), attributes);
                                writer.startUpdate(id);
                                encodeRow(writer, context, state);
                                writer.endUpdate();
                            }
                            catch(IOException e)
                            {
                                throw new FacesException(e);
                            }
                            return DataVisitResult.CONTINUE;
                        }

            
            {
                this$0 = ExtendedDataTableRenderer.this;
                state = rendererstate;
                writer = partialresponsewriter;
                ids = list;
                super();
            }
                    }, addRange, null);
                    writer.startEval();
                    if(difference < 0)
                    {
                        difference += clientRows;
                    }
                    writer.write((new StringBuilder()).append("var richTBody = document.getElementById('").append(clientId).append(":tb").append(partId).append("');").toString());
                    writer.write("var richRows = richTBody.rows;");
                    writer.write((new StringBuilder()).append("for (var i = 0; i < ").append(difference).append("; i++ ) richTBody.appendChild(richTBody.removeChild(richRows[0]));").toString());
                }

                writer.startUpdate((new StringBuilder()).append(clientId).append(":si").toString());
                encodeSelectionInput(writer, context, component);
                writer.endUpdate();
                writer.startEval();
                writer.write((new StringBuilder()).append("jQuery(").append(ScriptUtils.toScript((new StringBuilder()).append('#').append(ScriptUtils.escapeCSSMetachars(clientId)).toString())).append(").triggerHandler('rich:onajaxcomplete', {first: ").append(table.getClientFirst()).append("});").toString());
                writer.endEval();
                table.setRowKey(context, key);
                table.restoreOrigValue(context);
            }
            break MISSING_BLOCK_LABEL_976;
        }
        initialWriter = context.getResponseWriter();
        if(!$assertionsDisabled && (initialWriter instanceof OnOffResponseWriter))
        {
            throw new AssertionError();
        }
        context.setResponseWriter(new OnOffResponseWriter(initialWriter));
        RendererState state = createRowHolder(context, component, null);
        state.setEncoderVariance(EncoderVariance.partial);
        PartialResponseWriter writer = context.getPartialViewContext().getPartialResponseWriter();
        if("header".equals(metaComponentId))
        {
            encodeHeader(state);
            writer.startEval();
            writer.write((new StringBuilder()).append("jQuery(").append(ScriptUtils.toScript((new StringBuilder()).append('#').append(ScriptUtils.escapeCSSMetachars(table.getClientId(context))).toString())).append(").triggerHandler('rich:onajaxcomplete', {reinitializeHeader: true});").toString());
            writer.endEval();
        } else
        if("footer".equals(metaComponentId))
        {
            encodeFooter(state);
        } else
        if("body".equals(metaComponentId))
        {
            encodeBody(state);
            String clientId = table.getClientId(context);
            writer.startUpdate((new StringBuilder()).append(clientId).append(":si").toString());
            encodeSelectionInput(writer, context, component);
            writer.endUpdate();
            writer.startEval();
            writer.write((new StringBuilder()).append("jQuery(").append(ScriptUtils.toScript((new StringBuilder()).append('#').append(ScriptUtils.escapeCSSMetachars(clientId)).toString())).append(").triggerHandler('rich:onajaxcomplete', {first: ").append(table.getClientFirst()).append(", rowCount: ").append(getRowCount(component)).append(", reinitializeBody: true});").toString());
            writer.endEval();
        } else
        {
            throw new IllegalArgumentException((new StringBuilder()).append("Unsupported metaComponentIdentifier: ").append(metaComponentId).toString());
        }
        context.setResponseWriter(initialWriter);
        break MISSING_BLOCK_LABEL_976;
        Exception exception;
        exception;
        context.setResponseWriter(initialWriter);
        throw exception;
    }

    public void decodeMetaComponent(FacesContext context, UIComponent component, String metaComponentId)
    {
        throw new UnsupportedOperationException();
    }

    protected void doEncodeBegin(ResponseWriter writer, FacesContext context, UIComponent component)
        throws IOException
    {
        Map attributes = component.getAttributes();
        writer.startElement("div", component);
        writer.writeAttribute("id", component.getClientId(context), null);
        writer.writeAttribute("class", HtmlUtil.concatClasses(new Object[] {
            "rf-edt", (String)attributes.get("styleClass")
        }), null);
        RenderKitUtils.renderAttribute(context, "style", attributes.get("style"));
    }

    public RendererState createRowHolder(FacesContext context, UIComponent component, Object options[])
    {
        return new RendererState(context, (UIDataTableBase)component);
    }

    protected void doEncodeChildren(ResponseWriter writer, FacesContext context, UIComponent component)
        throws IOException
    {
        UIDataTableBase table = (UIDataTableBase)component;
        Object key = table.getRowKey();
        table.captureOrigValue(context);
        table.setRowKey(context, null);
        RendererState state = createRowHolder(context, table, null);
        encodeStyle(state);
        encodeHeader(state);
        encodeBody(state);
        encodeFooter(state);
        table.setRowKey(context, key);
        table.restoreOrigValue(context);
    }

    protected void doEncodeEnd(ResponseWriter writer, FacesContext context, UIComponent component)
        throws IOException
    {
        writer.startElement("table", component);
        String clientId = component.getClientId(context);
        writer.writeAttribute("id", (new StringBuilder()).append(clientId).append(":r").toString(), null);
        writer.writeAttribute("class", "rf-edt-rord", null);
        writer.writeAttribute("cellpadding", "0", null);
        writer.writeAttribute("cellspacing", "0", null);
        writer.startElement("tr", component);
        writer.startElement("th", component);
        writer.write("&#160;");
        writer.endElement("th");
        writer.endElement("tr");
        for(int i = 0; i < 6; i++)
        {
            writer.startElement("tr", component);
            writer.startElement("td", component);
            writer.write("&#160;");
            writer.endElement("td");
            writer.endElement("tr");
        }

        writer.endElement("table");
        writer.startElement("div", component);
        writer.writeAttribute("id", (new StringBuilder()).append(clientId).append(":d").toString(), null);
        writer.writeAttribute("class", "rf-edt-rsz-mkr", null);
        writer.endElement("div");
        writer.startElement("div", component);
        writer.writeAttribute("id", (new StringBuilder()).append(clientId).append(":rm").toString(), null);
        writer.writeAttribute("class", "rf-edt-rord-mkr", null);
        writer.endElement("div");
        writer.startElement("input", component);
        writer.writeAttribute("id", (new StringBuilder()).append(clientId).append(":wi").toString(), null);
        writer.writeAttribute("name", (new StringBuilder()).append(clientId).append(":wi").toString(), null);
        writer.writeAttribute("type", "hidden", null);
        writer.endElement("input");
        encodeSelectionInput(writer, context, component);
        AjaxFunction ajaxFunction = AjaxRendererUtils.buildAjaxFunction(context, component);
        ajaxFunction.getOptions().setClientParameters(CLIENT_PARAMS);
        Map attributes = component.getAttributes();
        Map options = new HashMap();
        RenderKitUtils.addToScriptHash(options, "selectionMode", attributes.get("selectionMode"), SelectionMode.multiple);
        RenderKitUtils.addToScriptHash(options, "onbeforeselectionchange", RenderKitUtils.getAttributeAndBehaviorsValue(context, component, (ComponentAttribute)EVENT_ATTRIBUTES.get("onbeforeselectionchange")), null, RenderKitUtils.ScriptHashVariableWrapper.eventHandler);
        RenderKitUtils.addToScriptHash(options, "onselectionchange", RenderKitUtils.getAttributeAndBehaviorsValue(context, component, (ComponentAttribute)EVENT_ATTRIBUTES.get("onselectionchange")), null, RenderKitUtils.ScriptHashVariableWrapper.eventHandler);
        StringBuilder builder = new StringBuilder("new RichFaces.ui.ExtendedDataTable('");
        builder.append(clientId).append("', ").append(getRowCount(component)).append(", function(event, clientParams) {").append(ajaxFunction.toScript()).append(";}");
        if(!options.isEmpty())
        {
            builder.append(",").append(ScriptUtils.toScript(options));
        }
        builder.append(");");
        getUtils().writeScript(context, component, builder.toString());
        writer.endElement("div");
    }

    private int getRowCount(UIComponent component)
    {
        UIDataTableBase table = (UIDataTableBase)component;
        int rows = table.getRows();
        int rowCount = table.getRowCount() - table.getFirst();
        if(rows > 0)
        {
            rows = Math.min(rows, rowCount);
        } else
        {
            rows = rowCount;
        }
        return rows;
    }

    private void encodeStyle(RendererState state)
        throws IOException
    {
        FacesContext context = state.getContext();
        ResponseWriter writer = context.getResponseWriter();
        UIDataTableBase table = state.getRow();
        PartialViewContext pvc = context.getPartialViewContext();
        if(!pvc.isAjaxRequest())
        {
            writer.startElement("style", table);
            writer.writeAttribute("type", "text/css", null);
            writer.writeText(getCSSText(context, table), null);
            writer.endElement("style");
        } else
        {
            writer.startElement("script", table);
            writer.writeAttribute("type", "text/javascript", null);
            String cssText = getCSSText(context, table);
            JSFunction function = new JSFunction("RichFaces.utils.addCSSText", new Object[] {
                cssText, (new StringBuilder()).append(table.getClientId(context)).append(":st").toString()
            });
            writer.writeText(function.toScript(), null);
            writer.endElement("script");
        }
    }

    protected String getCSSText(FacesContext context, UIDataTableBase table)
        throws IOException
    {
        StringBuilder sb = new StringBuilder();
        sb.append("div.rf-edt-cnt { width: 100%; }");
        for(Iterator columns = table.columns(); columns.hasNext(); sb.append("}"))
        {
            UIComponent column = (UIComponent)columns.next();
            String id = column.getId();
            if(id == null)
            {
                column.getClientId(context);
                id = column.getId();
            }
            String width = getColumnWidth(column);
            sb.append((new StringBuilder()).append(".rf-edt-c-").append(id).append(" {").toString());
            sb.append((new StringBuilder()).append("width: ").append(width).append(";").toString());
        }

        return sb.toString();
    }

    public void encodeRow(ResponseWriter writer, FacesContext facesContext, RowHolderBase rowHolder)
        throws IOException
    {
        RendererState state = (RendererState)rowHolder;
        AbstractExtendedDataTable table = (AbstractExtendedDataTable)state.getRow();
        writer.startElement("tr", table);
        if(rowHolder.getRow() instanceof UIDataTableBase)
        {
            renderRowHandlers(facesContext, (UIDataTableBase)rowHolder.getRow());
        }
        String rowClass = getRowClass(rowHolder);
        StringBuilder builder = new StringBuilder();
        Collection selection = table.getSelection();
        if(selection != null && selection.contains(table.getRowKey()))
        {
            builder.append("rf-edt-r-sel");
        }
        if(table.getRowKey().equals(table.getAttributes().get("activeRowKey")))
        {
            if(builder.length() > 0)
            {
                builder.append(' ');
            }
            builder.append("rf-edt-r-act");
        }
        if(table.getRowKey().equals(table.getAttributes().get("shiftRowKey")))
        {
            if(builder.length() > 0)
            {
                builder.append(' ');
            }
            builder.append("rf-edt-r-sht");
        }
        rowClass = concatClasses(new Object[] {
            builder.toString(), rowClass
        });
        if(rowClass.length() > 0)
        {
            writer.writeAttribute("class", rowClass, null);
        }
        Iterator columns = null;
        Part part = state.getPart();
        writer.writeAttribute("id", (new StringBuilder()).append(table.getContainerClientId(facesContext)).append(":").append(part.getName().getId()).toString(), null);
        columns = part.getColumns().iterator();
        int columnNumber = 0;
        do
        {
            if(!columns.hasNext())
            {
                break;
            }
            UIComponent column = (UIComponent)columns.next();
            if(column.isRendered())
            {
                writer.startElement("td", table);
                String columnClass = concatClasses(new Object[] {
                    getColumnClass(rowHolder, columnNumber), column.getAttributes().get("styleClass")
                });
                if(!"".equals(columnClass))
                {
                    writer.writeAttribute("class", columnClass, null);
                }
                String columnStyle = (String)column.getAttributes().get("style");
                if(!"".equals(columnStyle))
                {
                    writer.writeAttribute("style", columnStyle, null);
                }
                columnNumber++;
                writer.startElement("div", table);
                writer.writeAttribute("class", (new StringBuilder()).append("rf-edt-c rf-edt-c-").append(column.getId()).toString(), null);
                writer.startElement("div", column);
                writer.writeAttribute("class", "rf-edt-c-cnt", null);
                renderChildren(facesContext, column);
                writer.endElement("div");
                writer.endElement("div");
                writer.endElement("td");
            }
        } while(true);
        writer.endElement("tr");
    }

    protected void doDecode(FacesContext context, UIComponent component)
    {
        super.doDecode(context, component);
        Map map = context.getExternalContext().getRequestParameterMap();
        String clientId = component.getClientId(context);
        updateWidthOfColumns(context, component, (String)map.get((new StringBuilder()).append(clientId).append(":wi").toString()));
        if(map.get(clientId) != null)
        {
            updateColumnsOrder(context, component, (String)map.get("rich:columnsOrder"));
        }
        if(map.get(clientId) != null)
        {
            updateClientFirst(context, component, (String)map.get("rich:clientFirst"));
        }
        decodeSortingFiltering(context, component);
    }

    private void updateWidthOfColumns(FacesContext context, UIComponent component, String widthString)
    {
        if(widthString != null && widthString.length() > 0)
        {
            String widthArray[] = widthString.split(",");
            for(int i = 0; i < widthArray.length; i++)
            {
                String widthEntry[] = widthArray[i].split(":");
                UIComponent column = component.findComponent(widthEntry[0]);
                updateAttribute(context, column, "width", widthEntry[1]);
            }

        }
    }

    private void updateColumnsOrder(FacesContext context, UIComponent component, String columnsOrderString)
    {
        if(columnsOrderString != null && columnsOrderString.length() > 0)
        {
            String columnsOrder[] = columnsOrderString.split(",");
            updateAttribute(context, component, "columnsOrder", columnsOrder);
            context.getPartialViewContext().getRenderIds().add(component.getClientId(context));
        }
    }

    private void updateClientFirst(FacesContext context, UIComponent component, String clientFirst)
    {
        if(clientFirst != null && clientFirst.length() > 0)
        {
            Integer value = Integer.valueOf(clientFirst);
            Map attributes = component.getAttributes();
            if(!value.equals(attributes.get("clientFirst")))
            {
                attributes.put("submittedClientFirst", value);
                context.getPartialViewContext().getRenderIds().add((new StringBuilder()).append(component.getClientId(context)).append("@").append("scroll").toString());
            }
        }
    }


    public volatile RowHolderBase createRowHolder(FacesContext x0, UIComponent x1, Object x2[])
    {
        return createRowHolder(x0, x1, x2);
    }

}
